-- probably should break Refresh into Fill and Layout like maliceWindow so that we can update the layout without setting the data

-- Settings.TimeZoneOffset represents the number of hours to adjust local time to server time (either EST or EDT depending on DST)
-- so, in pacific time, Settings.TimeZoneOffset would be +3
-- in GMT, Settings.TimeZoneOffset would be -5
-- this allows the plugin to calculate 'day' based on servertime (minus 3 hours for the 3:00 AM reset time)
-- so at 2:59 AM on 6/5/2024, the day is 6/4/2024 (it hasn't switched to next day yet) but at 3:00 AM on 6/5/2024, the day is 6/5/2024

-- GetLocalTime seems to be Epochtime(UTC) adjusted for timezone and DST
-- since if we are not in the same time zone as the servers, we will only know if we are DST, not whether servers are also DST, that can cause a slight problem

-- create a window that displays the available missions for a given date
-- filterable by area (dropdown), NPC (dropdown), Mission Name (as string or pattern), DelvingOnly (checkbox)

-- really should generate this as a language specific set of names, but this will automatically pick up on new classes that are added to the Turbine.Gameplay.Class enumeration
className={}
for k,v in pairs(Turbine.Gameplay.Class) do
	className[v]=k
end
missionWindow = Turbine.UI.Lotro.Window()
missionWindow:SetZOrder(1)
missionWindow.MoveX=-1
missionWindow.MoveY=-1
missionWindow:SetSize(initMissionWidth,initMissionHeight)
missionWindow:SetPosition(initMissionLeft, initMissionTop )
missionWindow:SetVisible(false)
missionWindow:SetOpacity(Settings.Opacity)
missionWindow:SetText(Resource[language][35])
missionWindow:SetResizable(true)
fontMetric:SetFont(Settings.Font) -- just incase it is set to a different font (but it shouldn't be)
-- date, area, npc - note, missionDay gets recalculated when date or area are chosen as the schedule is determined by both
missionWindow.DateCaption=Turbine.UI.Label()
missionWindow.DateCaption:SetParent(missionWindow)
missionWindow.DateCaption:SetPosition(10,45)
missionWindow.DateCaption:SetSize(fontMetric:GetTextWidth(Resource[language][36]..":"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.DateCaption:SetForeColor(Settings.ForeColor)
missionWindow.DateCaption:SetBackColor(Settings.BackColor)
missionWindow.DateCaption:SetFont(Settings.Font)
missionWindow.DateCaption:SetText(Resource[language][36]..":")
missionWindow.DateCaption:SetMouseVisible(false)

--missionWindow.Date=Turbine.UI.Label()
missionWindow.Date=DateBox()
missionWindow.Date:SetParent(missionWindow)
missionWindow.Date:SetPosition(missionWindow.DateCaption:GetLeft()+missionWindow.DateCaption:GetWidth()+5,missionWindow.DateCaption:GetTop())
--missionWindow.Date:SetSize(fontMetric:GetTextWidth("88/88/8888x"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.Date:SetForeColor(Settings.ForeColor)
missionWindow.Date:SetBackColor(Settings.BackColor)
missionWindow.Date:SetFont(Settings.Font)
missionWindow.Date.Calendar:SetLanguage(language)
--missionWindow.Date:SetText(getCurrentDate(Settings.DateFormat))
do
	local date=Turbine.Engine:GetDate()
	missionWindow.Month=date.Month
	missionWindow.Day=date.Day
	missionWindow.Year=date.Year
	missionWindow.Date:SetDay(date.Day)
	missionWindow.Date:SetMonth(date.Month)
	missionWindow.Date:SetYear(date.Year)
end
missionWindow.Date.DateChanged=function()
	missionWindow:Refresh(true)
end
--[[missionWindow.Calendar=Calendar()
missionWindow.Calendar:SetZOrder(missionWindow:GetZOrder()+1)
missionWindow.Date.MouseDoubleClick=function(sender,args)
	missionWindow.Calendar:SetMonth(missionWindow.Month)
	missionWindow.Calendar:SetDay(missionWindow.Day)
	missionWindow.Calendar:SetYear(missionWindow.Year)
	missionWindow.Calendar.Callback=missionWindow.DateSelected
	local left,top=missionWindow:PointToScreen(args.X,args.Y)
	missionWindow.Calendar:SetPosition(left,top)
	missionWindow.Calendar:SetVisible(true)
end
missionWindow.DateSelected=function(args)
	missionWindow.Day=args.Day
	missionWindow.Month=args.Month
	missionWindow.Year=args.Year
	local newDate=string.gsub(Settings.DateFormat,"YYYY",string.format("%04d",args.Year))
	newDate=string.gsub(newDate,"DD",string.format("%02d",args.Day))
	missionWindow.Date:SetText(string.gsub(newDate,"MM",string.format("%02d",args.Month)))
end
]]
missionWindow.Time=Turbine.UI.Label()
missionWindow.Time:SetParent(missionWindow)
missionWindow.Time:SetPosition(missionWindow.Date:GetLeft()+missionWindow.Date:GetWidth()+5,missionWindow.Date:GetTop())
missionWindow.Time:SetSize(fontMetric:GetTextWidth("88:88:88 "),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
--missionWindow.Time:SetBackColor(Settings.BackColor)
missionWindow.Time:SetMultiline(false)
missionWindow.Time:SetForeColor(Settings.ForeColor)
missionWindow.Time:SetBackColor(Settings.BackColor)
missionWindow.Time:SetFont(Settings.Font)
missionWindow.AreaCaption=Turbine.UI.Label()
missionWindow.AreaCaption:SetParent(missionWindow)
missionWindow.AreaCaption:SetPosition(missionWindow.Time:GetLeft()+missionWindow.Time:GetWidth()+10,missionWindow.Date:GetTop())
missionWindow.AreaCaption:SetSize(fontMetric:GetTextWidth(Resource[language][37]..":"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.AreaCaption:SetFont(Settings.Font)
missionWindow.AreaCaption:SetText(Resource[language][37]..":")
missionWindow.AreaCaption:SetMouseVisible(false)

missionWindow.Area=DropDownList()
missionWindow.Area:SetParent(missionWindow)
missionWindow.Area:SetBorderColor(Settings.TrimColor)
missionWindow.Area:SetTextColor(Settings.ForeColor)
missionWindow.Area:SetBackColor(Settings.BackColor)
missionWindow.Area:SetCurrentBackColor(Settings.BackColor)
missionWindow.Area:SetFont(Settings.Font)
missionWindow.Area:SetZOrder(1)
missionWindow.Area:SetPosition(missionWindow.AreaCaption:GetLeft()+missionWindow.AreaCaption:GetWidth()+5,missionWindow.AreaCaption:GetTop()-1)
-- determine widest option
do
	local longest=1
	local tmpLength=0
	for k,v in ipairs(MissionArea) do
		missionWindow.Area:AddItem(v[1],k)
		if string.len(v[1])>tmpLength then
			tmpLength=string.len(v[1])
			longest=k
		end
	end
	missionWindow.Area:SetSize(fontMetric:GetTextWidth(MissionArea[longest][1])+25,Turbine.UI.Lotro.FontInfo[Settings.Font].size+2)
end
missionWindow.Area.SelectedIndexChanged=function()
	missionWindow.NPCList:Refresh()
end

missionWindow.NPCCaption=Turbine.UI.Label()
missionWindow.NPCCaption:SetParent(missionWindow)
missionWindow.NPCCaption:SetPosition(missionWindow.Area:GetLeft()+missionWindow.Area:GetWidth()+10,missionWindow.Date:GetTop())
missionWindow.NPCCaption:SetSize(fontMetric:GetTextWidth(Resource[language][38]..":"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.NPCCaption:SetForeColor(Settings.ForeColor)
missionWindow.NPCCaption:SetBackColor(Settings.BackColor)
missionWindow.NPCCaption:SetFont(Settings.Font)
missionWindow.NPCCaption:SetText(Resource[language][38]..":")
missionWindow.NPCCaption:SetMouseVisible(false)

missionWindow.NPCList=DropDownList()
missionWindow.NPCList:SetParent(missionWindow)
missionWindow.NPCList:SetBorderColor(Settings.TrimColor)
missionWindow.NPCList:SetTextColor(Settings.ForeColor)
missionWindow.NPCList:SetBackColor(Settings.BackColor)
missionWindow.NPCList:SetCurrentBackColor(Settings.BackColor)
missionWindow.NPCList:SetZOrder(1)
missionWindow.NPCList:SetFont(Settings.Font)
missionWindow.NPCList:SetPosition(missionWindow.NPCCaption:GetLeft()+missionWindow.NPCCaption:GetWidth()+5,missionWindow.NPCCaption:GetTop()-1)

missionWindow.missionCurrentDay=Turbine.UI.Label()
missionWindow.missionCurrentDay:SetParent(missionWindow)
missionWindow.missionCurrentDay:SetFont(Settings.Font)
missionWindow.missionCurrentDay:SetSize(20,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.missionCurrentDay:SetForeColor(Settings.ForeColor)
missionWindow.missionCurrentDay:SetBackColor(Settings.BackColor)
missionWindow.missionCurrentDay:SetBackColor(Turbine.UI.Color.Blue)
--missionWindow.missionCurrentDay:SetVisible(false)

missionWindow.NPCList.Refresh=function()
	local area=missionWindow.Area:GetSelectedIndex()
	if area==nil then area=1 end
	local longest=1
	local tmpLength=0
	missionWindow.NPCList:ClearList()
	for k,v in ipairs(MissionNPC[area]) do
		missionWindow.NPCList:AddItem(v[1],k)
		if string.len(v[1])>tmpLength then
			tmpLength=string.len(v[1])
			longest=k
		end
	end
	missionWindow.NPCList:SetSize(fontMetric:GetTextWidth(MissionNPC[area][longest][1])+25,Turbine.UI.Lotro.FontInfo[Settings.Font].size+2)
	missionWindow.missionCurrentDay:SetPosition(missionWindow.NPCList:GetLeft()+missionWindow.NPCList:GetWidth()+5,missionWindow.NPCList:GetTop())
	missionWindow.NPCList:SetSelectedIndex(1)
end
missionWindow.NPCList.SelectedIndexChanged=function()
	missionWindow:Refresh(true)
end

missionWindow.MissionListBack=Turbine.UI.Control()
missionWindow.MissionListBack:SetParent(missionWindow)
missionWindow.MissionListBack:SetPosition(10,missionWindow.DateCaption:GetTop()+missionWindow.DateCaption:GetHeight()+10)
missionWindow.MissionListBack:SetSize(missionWindow:GetWidth()-20,missionWindow:GetHeight()-45-missionWindow.MissionListBack:GetTop())
missionWindow.MissionListBack:SetBackColor(Settings.TrimColor)
missionWindow.MissionListBack:SetMouseVisible(false)

missionWindow.MissionList=Turbine.UI.ListBox()
missionWindow.MissionList:SetMouseVisible(false)

missionWindow.MissionListCaptBack=Turbine.UI.Control()
missionWindow.MissionListCaptBack:SetParent(missionWindow.MissionListBack)
missionWindow.MissionListCaptBack:SetPosition(1,1)
missionWindow.MissionListCaptBack:SetBackColor(Settings.BackColor)
missionWindow.MissionListCaptBack:SetWidth(missionWindow.MissionListBack:GetWidth()-2)
missionWindow.MissionListCaptBack:SetHeight(Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.MissionListCaptBack:SetMouseVisible(false)

missionWindow.MissionListCaptions=Turbine.UI.Control()
missionWindow.MissionListCaptions:SetParent(missionWindow.MissionListCaptBack)
missionWindow.MissionListCaptions:SetBackColor(Settings.TrimColor)
missionWindow.MissionListCaptions:SetPosition(0,0)
if missionWindow.MissionList:GetWidth()>602 then
	missionWindow.MissionListCaptions:SetSize(missionWindow.MissionList:GetWidth()-302,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
else
	missionWindow.MissionListCaptions:SetSize(602,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
end
missionWindow.MissionListCaptions:SetHeight(Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.MissionListCaptions:SetMouseVisible(false)

missionWindow.MissionListDelvCaption=Turbine.UI.Label()
missionWindow.MissionListDelvCaption:SetForeColor(Settings.ForeColor)
missionWindow.MissionListDelvCaption:SetBackColor(Settings.BackColor)
missionWindow.MissionListDelvCaption:SetParent(missionWindow.MissionListCaptions)
missionWindow.MissionListDelvCaption:SetFont(Settings.Font)
missionWindow.MissionListDelvCaption:SetPosition(0,0)
missionWindow.MissionListDelvCaption:SetSize(100,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.MissionListDelvCaption:SetText("Delving")
missionWindow.MissionListDelvCaption:SetMouseVisible(false)

missionWindow.MissionListNameCaption=Turbine.UI.Label()
missionWindow.MissionListNameCaption:SetBackColor(Settings.BackColor)
missionWindow.MissionListNameCaption:SetParent(missionWindow.MissionListCaptions)
missionWindow.MissionListNameCaption:SetFont(Settings.Font)
missionWindow.MissionListNameCaption:SetPosition(missionWindow.MissionListDelvCaption:GetLeft()+missionWindow.MissionListDelvCaption:GetWidth()+1,missionWindow.MissionListDelvCaption:GetTop())
missionWindow.MissionListNameCaption:SetSize(300,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.MissionListNameCaption:SetText("Name")
missionWindow.MissionListNameCaption:SetMouseVisible(false)

missionWindow.MissionListDescCaption=Turbine.UI.Label()
missionWindow.MissionListDescCaption:SetBackColor(Settings.BackColor)
missionWindow.MissionListDescCaption:SetParent(missionWindow.MissionListCaptions)
missionWindow.MissionListDescCaption:SetFont(Settings.Font)
missionWindow.MissionListDescCaption:SetPosition(missionWindow.MissionListNameCaption:GetLeft()+missionWindow.MissionListNameCaption:GetWidth()+1,missionWindow.MissionListNameCaption:GetTop())
missionWindow.MissionListDescCaption:SetSize(missionWindow.MissionListCaptions:GetWidth()-302,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
missionWindow.MissionListDescCaption:SetText("Description")
missionWindow.MissionListDescCaption:SetMouseVisible(false)

missionWindow.MissionList:SetParent(missionWindow.MissionListBack)
missionWindow.MissionList:SetPosition(1,missionWindow.MissionListCaptBack:GetTop()+missionWindow.MissionListCaptBack:GetHeight()+1)
missionWindow.MissionList:SetSize(missionWindow.MissionListBack:GetWidth()-12,missionWindow.MissionListBack:GetHeight()-missionWindow.MissionList:GetTop()-12)
missionWindow.MissionList:SetBackColor(Settings.BackColor)
missionWindow.MissionVScrollBack=Turbine.UI.Control()
missionWindow.MissionVScrollBack:SetParent(missionWindow.MissionListBack)
missionWindow.MissionVScrollBack:SetPosition(missionWindow.MissionList:GetLeft()+missionWindow.MissionList:GetWidth(),missionWindow.MissionList:GetTop())
missionWindow.MissionVScrollBack:SetSize(10,missionWindow.MissionList:GetHeight()+10)
missionWindow.MissionVScrollBack:SetBackColor(Settings.BackColor)
missionWindow.MissionVScroll=Turbine.UI.Lotro.ScrollBar()
missionWindow.MissionVScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
missionWindow.MissionVScroll:SetParent(missionWindow.MissionListBack)
missionWindow.MissionVScroll:SetBackColor(Settings.BackColor)
missionWindow.MissionVScroll:SetWidth(10)
missionWindow.MissionVScroll:SetHeight(missionWindow.MissionList:GetHeight())
missionWindow.MissionVScroll:SetPosition(missionWindow.MissionList:GetLeft()+missionWindow.MissionList:GetWidth(),missionWindow.MissionList:GetTop())
missionWindow.MissionList:SetVerticalScrollBar(missionWindow.MissionVScroll)

missionWindow.MissionHScrollBack=Turbine.UI.Control()
missionWindow.MissionHScrollBack:SetParent(missionWindow.MissionListBack)
missionWindow.MissionHScrollBack:SetPosition(missionWindow.MissionList:GetLeft(),missionWindow.MissionList:GetTop()+missionWindow.MissionList:GetHeight())
missionWindow.MissionHScrollBack:SetSize(missionWindow.MissionList:GetWidth(),10)
missionWindow.MissionHScrollBack:SetBackColor(Settings.BackColor)
missionWindow.MissionHScroll=Turbine.UI.Lotro.ScrollBar()
missionWindow.MissionHScroll:SetOrientation(Turbine.UI.Orientation.Horizontal)
missionWindow.MissionHScroll:SetParent(missionWindow.MissionListBack)
missionWindow.MissionHScroll:SetBackColor(Settings.BackColor)
missionWindow.MissionHScroll:SetHeight(10)
missionWindow.MissionHScroll:SetWidth(missionWindow.MissionList:GetWidth())
missionWindow.MissionHScroll:SetPosition(missionWindow.MissionList:GetLeft(),missionWindow.MissionList:GetTop()+missionWindow.MissionList:GetHeight())
missionWindow.MissionList:SetHorizontalScrollBar(missionWindow.MissionHScroll)

-- need handler for position changed for first row of missionList, so we will make a row of height 0 and add it so we always have it - if we clear the list, we need to re-add row0
missionRow0=Turbine.UI.Control()
missionRow0:SetParent(missionWindow.MissionList)
missionRow0:SetSize(missionWindow.MissionList:GetWidth(),0)
missionRow0.PositionChanged=function(sender,args)
	missionWindow.MissionListCaptions:SetLeft(missionRow0:GetLeft())
end
missionRow0:SetMouseVisible(false)
missionWindow.MissionList:AddItem(missionRow0)

missionWindow.Refresh=function(forceRefresh)
	missionWindow.MissionListBack:SetSize(missionWindow:GetWidth()-20,missionWindow:GetHeight()-45-missionWindow.MissionListBack:GetTop())
	missionWindow.MissionListCaptBack:SetWidth(missionWindow.MissionListBack:GetWidth()-2)
	missionWindow.MissionList:SetTop(missionWindow.MissionListCaptBack:GetTop()+missionWindow.MissionListCaptBack:GetHeight()+1)
	missionWindow.MissionList:SetSize(missionWindow.MissionListBack:GetWidth()-12,missionWindow.MissionListBack:GetHeight()-missionWindow.MissionList:GetTop()-12)
	if missionWindow.MissionList:GetWidth()>missionWindow.MissionListDescCaption:GetLeft()+300 then
--		missionWindow.MissionListCaptions:SetSize(missionWindow.MissionList:GetWidth()-302,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
		missionWindow.MissionListCaptions:SetSize(missionWindow.MissionList:GetWidth(),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	else
		missionWindow.MissionListCaptions:SetSize(missionWindow.MissionListDescCaption:GetLeft()+300,Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	end
	missionWindow.MissionListDescCaption:SetSize(missionWindow.MissionListCaptions:GetWidth()-missionWindow.MissionListDescCaption:GetLeft(),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.MissionList:ClearItems()
	missionRow0:SetWidth(missionWindow.MissionListCaptions:GetWidth())
	missionWindow.MissionVScrollBack:SetPosition(missionWindow.MissionList:GetLeft()+missionWindow.MissionList:GetWidth(),missionWindow.MissionList:GetTop())
	missionWindow.MissionVScrollBack:SetSize(10,missionWindow.MissionList:GetHeight()+10)
	missionWindow.MissionVScroll:SetHeight(missionWindow.MissionList:GetHeight())
	missionWindow.MissionVScroll:SetPosition(missionWindow.MissionList:GetLeft()+missionWindow.MissionList:GetWidth(),missionWindow.MissionList:GetTop())
	missionWindow.MissionHScrollBack:SetPosition(missionWindow.MissionList:GetLeft(),missionWindow.MissionList:GetTop()+missionWindow.MissionList:GetHeight())
	missionWindow.MissionHScrollBack:SetSize(missionWindow.MissionList:GetWidth(),10)
	missionWindow.MissionHScroll:SetWidth(missionWindow.MissionList:GetWidth())
	missionWindow.MissionHScroll:SetPosition(missionWindow.MissionList:GetLeft(),missionWindow.MissionList:GetTop()+missionWindow.MissionList:GetHeight())

	missionRow0:SetLeft(0)
	missionWindow.MissionList:AddItem(missionRow0) -- need to ALWAYS re-add this item
	missionWindow.MissionListCaptions:SetLeft(0)

-- reload the data based on the currently selected date, area and NPC
	local area=missionWindow.Area:GetSelectedIndex()
	local npc=missionWindow.NPCList:GetSelectedIndex()
	local daysInCycle=MissionArea[area][2]
	local scheduleFunction=MissionArea[area][3]
	local scheduleDayOne=MissionArea[area][4] -- no longer used since all schedules are set based on start day 1/1/1970
	if scheduleDayOne==nil then scheduleDayOne=0 end
	local currentDateTime=Turbine.Engine.GetDate()
--	if currentDateTime.Hour+Settings.TimeZoneOffset-3>24 then
--		missionWindow.Time:SetForeColor(Turbine.UI.Color.Orange)
--	else
		missionWindow.Time:SetForeColor(Settings.ForeColor)
--	end
	missionWindow.Time:SetText(string.format("%02d:%02d:%02d",currentDateTime.Hour,currentDateTime.Minute,currentDateTime.Second))
	local currentDay=1
if Settings.Debug then Turbine.Shell.WriteLine("daysInCycle:"..tostring(daysInCycle)) end
	if daysInCycle~=nil and daysInCycle>1 then
		local resetTime=3600*3 -- we subtract this from the time to allow for the reset at 03:00 servertime
		local dateTime=Turbine.Engine.GetDate()
		dateTime.Month=missionWindow.Date.Month
		dateTime.Day=missionWindow.Date.Day
		dateTime.Year=missionWindow.Date.Year

--		local _,tmpVariant=math.modf(math.floor(epochDate/86400)/6) -- floor(date/86400)  is days since 1/1/1970 - zero based - 86400 seconds per day divided by days in cycle to get remainder which is fractional part of current cycle

		local _,tmpDay=math.modf(math.floor((getEpochTime(dateTime)-Settings.TimeZoneOffset*3600-resetTime)/86400)/daysInCycle)
if Settings.Debug then Turbine.Shell.WriteLine("epochtime:"..tostring(Turbine.Engine:GetLocalTime()).." calc time:"..tostring(getEpochTime(dateTime))) end
if Settings.Debug then Turbine.Shell.WriteLine("difference:"..tostring(Turbine.Engine:GetLocalTime()-getEpochTime(dateTime))) end
if Settings.Debug then Turbine.Shell.WriteLine("tmpDay1:"..tostring(tmpDay)) end
		local tmpDay=math.floor(tmpDay*daysInCycle+.1)+1
if Settings.Debug then Turbine.Shell.WriteLine("tmpDay2:"..tostring(tmpDay)) end
		currentDay=scheduleFunction(tmpDay)
	end
	if forceRefresh or missionWindow.missionCurrentDay:GetText()~=tostring(currentDay) then
		missionWindow.missionCurrentDay:SetText(tostring(currentDay)) -- only for debugging
		if MissionGroup[area]~=nil and MissionGroup[area][npc]~=nil and MissionGroup[area][npc][currentDay]~=nil then
			local missionGroup=MissionGroup[area][npc][currentDay]
			for k,v in ipairs(missionGroup) do
				local tmpRow=Turbine.UI.Control()
				tmpRow:SetMouseVisible(false)
				local height=fontMetric:GetTextHeight(MissionData[v][2],missionWindow.MissionListNameCaption:GetWidth())
				local height2=fontMetric:GetTextHeight(MissionData[v][3],missionWindow.MissionListDescCaption:GetWidth())
				if height2>height then height=height2 end
				tmpRow:SetBackColor(Settings.TrimColor)
				tmpRow:SetSize(missionWindow.MissionListCaptions:GetWidth(),height+1)
				tmpRow:SetParent(missionWindow.MissionList)
				tmpRow.Delving=Turbine.UI.Label()
				tmpRow.Delving:SetParent(tmpRow)
				tmpRow.Delving:SetBackColor(Settings.BackColor)
				tmpRow.Delving:SetFont(Settings.Font)
				tmpRow.Delving:SetSize(missionWindow.MissionListDelvCaption:GetWidth(),height)
				if MissionData[v][1] then
					tmpRow.Delving:SetText("Y")
				end
				tmpRow.Delving:SetMouseVisible(false)
				tmpRow.Name=Turbine.UI.Label()
				tmpRow.Name:SetParent(tmpRow)
				tmpRow.Name:SetBackColor(Settings.BackColor)
				tmpRow.Name:SetFont(Settings.Font)
				tmpRow.Name:SetSize(missionWindow.MissionListNameCaption:GetWidth(),height)
				tmpRow.Name:SetLeft(missionWindow.MissionListNameCaption:GetLeft())
				tmpRow.Name:SetText(MissionData[v][2])
				tmpRow.Name:SetMouseVisible(false)

				tmpRow.Desc=Turbine.UI.Label()
				tmpRow.Desc:SetParent(tmpRow)
				tmpRow.Desc:SetBackColor(Settings.BackColor)
				tmpRow.Desc:SetFont(Settings.Font)
				tmpRow.Desc:SetSize(missionWindow.MissionListDescCaption:GetWidth(),height)
				tmpRow.Desc:SetLeft(missionWindow.MissionListDescCaption:GetLeft())
				local tmpText=string.gsub(MissionData[v][3],"%@CharName",localPlayer:GetName())
				tmpText=string.gsub(tmpText,"%@Class",className[localPlayer:GetClass()])
				tmpRow.Desc:SetText(tmpText)
				tmpRow.Desc:SetMouseVisible(false)
				missionWindow.MissionList:AddItem(tmpRow)
			end
		end
	end
end
missionWindow.Update=function(sender,args)
	if missionWindow.updateTime==nil or Turbine.Engine:GetGameTime()>missionWindow.updateTime then
		missionWindow:Refresh(false)
		missionWindow.updateTime=Turbine.Engine:GetGameTime()+1
	end
end
missionWindow.SetFont=function()
	-- need to resize/layout all controls
	missionWindow.DateCaption:SetSize(fontMetric:GetTextWidth(Resource[language][36]..":"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.DateCaption:SetFont(Settings.Font)
	missionWindow.DateCaption:SetText(missionWindow.DateCaption:GetText())
	missionWindow.Date:SetPosition(missionWindow.DateCaption:GetLeft()+missionWindow.DateCaption:GetWidth()+5,missionWindow.DateCaption:GetTop())
	missionWindow.Date:SetSize(fontMetric:GetTextWidth("88/88/8888x"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.Date:SetFont(Settings.Font)
	missionWindow.Time:SetPosition(missionWindow.Date:GetLeft()+missionWindow.Date:GetWidth()+5,missionWindow.DateCaption:GetTop())
	missionWindow.Time:SetSize(fontMetric:GetTextWidth("88:88:88 "),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.Time:SetFont(Settings.Font)
	missionWindow.AreaCaption:SetPosition(missionWindow.Time:GetLeft()+missionWindow.Time:GetWidth()+10,missionWindow.Date:GetTop())
	missionWindow.AreaCaption:SetSize(fontMetric:GetTextWidth(Resource[language][37]..":"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.AreaCaption:SetFont(Settings.Font)
	missionWindow.AreaCaption:SetText(missionWindow.AreaCaption:GetText())
	missionWindow.Area:SetFont(Settings.Font)
	missionWindow.Area:SetPosition(missionWindow.AreaCaption:GetLeft()+missionWindow.AreaCaption:GetWidth()+5,missionWindow.AreaCaption:GetTop()-1)
	-- determine widest option
	local longest=1
	local tmpLength=0
	for k,v in ipairs(MissionArea) do
		if string.len(v[1])>tmpLength then
			tmpLength=string.len(v[1])
			longest=k
		end
	end
	missionWindow.Area:SetSize(fontMetric:GetTextWidth(MissionArea[longest][1])+25,Turbine.UI.Lotro.FontInfo[Settings.Font].size+2)
	missionWindow.NPCCaption:SetPosition(missionWindow.Area:GetLeft()+missionWindow.Area:GetWidth()+10,missionWindow.Date:GetTop())
	missionWindow.NPCCaption:SetSize(fontMetric:GetTextWidth(Resource[language][38]..":"),Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.NPCCaption:SetFont(Settings.Font)
	missionWindow.NPCCaption:SetText(missionWindow.NPCCaption:GetText())
	missionWindow.NPCList:SetFont(Settings.Font)
	missionWindow.NPCList:SetPosition(missionWindow.NPCCaption:GetLeft()+missionWindow.NPCCaption:GetWidth()+5,missionWindow.NPCCaption:GetTop()-1)
	missionWindow.MissionListBack:SetPosition(10,missionWindow.DateCaption:GetTop()+missionWindow.DateCaption:GetHeight()+10)
	local area=missionWindow.Area:GetSelectedIndex()
	if area==nil then area=1 end
	longest=1
	tmpLength=0
	for k,v in ipairs(MissionNPC[area]) do
		if string.len(v[1])>tmpLength then
			tmpLength=string.len(v[1])
			longest=k
		end
	end
	missionWindow.NPCList:SetSize(fontMetric:GetTextWidth(MissionNPC[area][longest][1])+25,Turbine.UI.Lotro.FontInfo[Settings.Font].size+2)
	missionWindow.missionCurrentDay:SetPosition(missionWindow.NPCList:GetLeft()+missionWindow.NPCList:GetWidth()+5,missionWindow.NPCList:GetTop())
	missionWindow.MissionListDelvCaption:SetFont(Settings.Font)
	missionWindow.MissionListDelvCaption:SetHeight(Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.MissionListDelvCaption:SetText(missionWindow.MissionListDelvCaption:GetText())
	missionWindow.MissionListNameCaption:SetFont(Settings.Font)
	missionWindow.MissionListNameCaption:SetHeight(Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.MissionListNameCaption:SetText(missionWindow.MissionListNameCaption:GetText())
	missionWindow.MissionListDescCaption:SetFont(Settings.Font)
	missionWindow.MissionListDescCaption:SetHeight(Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.MissionListDescCaption:SetText(missionWindow.MissionListDescCaption:GetText())
	missionWindow.MissionListCaptions:SetHeight(Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow.MissionListCaptBack:SetHeight(Turbine.UI.Lotro.FontInfo[Settings.Font].size)
	missionWindow:Refresh(true)

--	missionWindow.Calendar:SetFont(Settings.Font)
end
missionWindow.Area:SetSelectedIndex(1)
--missionWindow.Refresh()
missionWindow.SizeChanged=function()
	missionWindow.Refresh(true)
end
missionWindow.MouseDown=function()
	missionWindow.Date.Calendar:SetVisible(false)
end
missionWindow.Closing=function()
	missionWindow.Date.Calendar:SetVisible(false)
end

missionWindow.VisibleChanged=function()
	missionWindow:SetWantsUpdates(missionWindow:IsVisible())
end

initMissionLeft=nil
initMissionTop=nil
initMissionWidth=nil
initMissionHeight=nil
